import { useEffect, useRef } from "react";
import initDemo from "./util/initDemo.js";
import {
  RenderingEngine,
  Types,
  Enums,
  setVolumesForViewports,
  volumeLoader,
  getRenderingEngine,
} from "@cornerstonejs/core";
import * as cornerstoneTools from "@cornerstonejs/tools";
import createImageIdsAndCacheMetaData from "./util/createImageIdsAndCacheMetaData";
import { PublicViewportInput } from "@cornerstonejs/core/dist/types/types/IViewport.js";
import setCtTransferFunctionForVolumeActor from "./util/setCtTransferFunctionForVolumeActor";
import * as cornerstone from "@cornerstonejs/core";

const {
  ToolGroupManager,
  Enums: csToolsEnums,
  CrosshairsTool,
  StackScrollMouseWheelTool,
} = cornerstoneTools;

const { ViewportType } = Enums;

interface CrosshairMprProps {
  children?: JSX.Element;
}

const viewportId1 = "CT_AXIAL";
const viewportId2 = "CT_SAGITTAL";
const viewportId3 = "CT_CORONAL";
const toolGroupId = "group";

const viewportColors = {
  [viewportId1]: "rgb(200, 0, 0)",
  [viewportId2]: "rgb(200, 200, 0)",
  [viewportId3]: "rgb(0, 200, 0)",
};

const viewportReferenceLineControllable = [
  viewportId1,
  viewportId2,
  viewportId3,
];

const viewportReferenceLineDraggableRotatable = [
  viewportId1,
  viewportId2,
  viewportId3,
];

const viewportReferenceLineSlabThicknessControlsOn = [
  viewportId1,
  viewportId2,
  viewportId3,
];

// Define a unique id for the volume
const volumeName = "CT_VOLUME_ID"; // Id of the volume less loader prefix
const volumeLoaderScheme = "cornerstoneStreamingImageVolume"; // Loader id which defines which volume loader to use
const volumeId = `${volumeLoaderScheme}:${volumeName}`; // VolumeId with loader id + volume id

function getReferenceLineColor(viewportId) {
  return viewportColors[viewportId];
}

function getReferenceLineControllable(viewportId) {
  const index = viewportReferenceLineControllable.indexOf(viewportId);
  return index !== -1;
}

function getReferenceLineDraggableRotatable(viewportId) {
  const index = viewportReferenceLineDraggableRotatable.indexOf(viewportId);
  return index !== -1;
}

function getReferenceLineSlabThicknessControlsOn(viewportId) {
  const index =
    viewportReferenceLineSlabThicknessControlsOn.indexOf(viewportId);
  return index !== -1;
}

const renderingEngineId = "myRenderingEngine";

export const CrosshairMpr = (props: CrosshairMprProps) => {
  const viewportId1Ref = useRef(null);
  const viewportId2Ref = useRef(null);
  const viewportId3Ref = useRef(null);
  const renderingEngine = useRef<RenderingEngine>();
  const imageIds = useRef<string[]>();

  const run = async () => {
    initDemo();

    // Get Cornerstone imageIds for the source data and fetch metadata into RAM
    imageIds.current = await createImageIdsAndCacheMetaData({
      StudyInstanceUID:
        "1.2.840.113564.345049290535.9692.637552042156233117.433089",
      SeriesInstanceUID:
        "1.3.12.2.1107.5.1.4.76315.30000021042706150001900118311",
      wadoRsRoot: "/dicom-web",
    });

    // Define a volume in memory
    const volume = await volumeLoader.createAndCacheVolume(volumeId, {
      imageIds: imageIds.current ?? [],
    });

    // Instantiate a rendering engine
    renderingEngine.current = new RenderingEngine(renderingEngineId);

    // Create the viewports
    const viewportInputArray: PublicViewportInput[] = [
      {
        viewportId: viewportId1,
        type: ViewportType.ORTHOGRAPHIC,
        element: viewportId1Ref.current!,
        defaultOptions: {
          orientation: Enums.OrientationAxis.AXIAL,
          background: [0, 0, 0],
        },
      },
      {
        viewportId: viewportId2,
        type: ViewportType.ORTHOGRAPHIC,
        element: viewportId2Ref.current!,
        defaultOptions: {
          orientation: Enums.OrientationAxis.SAGITTAL,
          background: [0, 0, 0],
        },
      },
      {
        viewportId: viewportId3,
        type: ViewportType.ORTHOGRAPHIC,
        element: viewportId3Ref.current!,
        defaultOptions: {
          orientation: Enums.OrientationAxis.CORONAL,
          background: [0, 0, 0],
        },
      },
    ];

    renderingEngine.current.setViewports(viewportInputArray);

    console.log(volume);

    // Set the volume to load
    volume.load();

    // Set volumes on the viewports
    await setVolumesForViewports(
      renderingEngine.current,
      [
        {
          volumeId,
          callback: setCtTransferFunctionForVolumeActor,
        },
      ],
      [viewportId1, viewportId2, viewportId3]
    );

    // Define tool groups to add the segmentation display tool to
    const toolGroup = ToolGroupManager.createToolGroup(toolGroupId);

    if (toolGroup) {
      // For the crosshairs to operate, the viewports must currently be
      // added ahead of setting the tool active. This will be improved in the future.
      toolGroup.addViewport(viewportId1, renderingEngineId);
      toolGroup.addViewport(viewportId2, renderingEngineId);
      toolGroup.addViewport(viewportId3, renderingEngineId);

      // Manipulation Tools
      toolGroup.addTool(StackScrollMouseWheelTool.toolName);
      // Add Crosshairs tool and configure it to link the three viewports
      // These viewports could use different tool groups. See the PET-CT example
      // for a more complicated used case.

      toolGroup.addTool(CrosshairsTool.toolName, {
        getReferenceLineColor,
        getReferenceLineControllable,
        getReferenceLineDraggableRotatable,
        getReferenceLineSlabThicknessControlsOn,
      });

      toolGroup.setToolActive(CrosshairsTool.toolName, {
        bindings: [{ mouseButton: 1 }],
      });
      // As the Stack Scroll mouse wheel is a tool using the `mouseWheelCallback`
      // hook instead of mouse buttons, it does not need to assign any mouse button.
      toolGroup.setToolActive(StackScrollMouseWheelTool.toolName);
    }

    renderingEngine.current.renderViewports([
      viewportId1,
      viewportId2,
      viewportId3,
    ]);
  };

  useEffect(() => {
    cornerstoneTools.addTool(StackScrollMouseWheelTool);
    cornerstoneTools.addTool(CrosshairsTool);
    console.log("添加tool1");

    run();

    return () => {
      // TODO: 是否需要写在组件，根据页面需要
      console.log("卸载tool2");
      cornerstoneTools.removeTool(StackScrollMouseWheelTool);
      cornerstoneTools.removeTool(CrosshairsTool);
    };
  }, []);

  useEffect(() => {
    window.addEventListener("resize", () => {
      renderingEngine.current?.resize();
    });
  }, []);

  const test = () => {
    // console.log(volumeLoader)
    console.log(cornerstone.cache.getVolume(volumeId).getScalarData());
  };

  useEffect(() => {
    document.addEventListener(Enums.Events.VOLUME_CACHE_VOLUME_ADDED, () => {
      console.log(111);
    });
  }, []);

  return (
    <div>
      <div style={{ display: "flex", gap: 20 }}>
        <div style={{ width: 300, height: 300 }} ref={viewportId1Ref}></div>
        <div style={{ width: 300, height: 300 }} ref={viewportId2Ref}></div>
        <div style={{ width: 300, height: 300 }} ref={viewportId3Ref}></div>
      </div>
      <button onClick={test}>测试</button>
    </div>
  );
};
